@@ -42,7 +42,7 @@ def _get_dash_pattern(style):
     # dashed styles
     elif style in ['dashed', 'dashdot', 'dotted']:
         offset = 0
-        dashes = tuple(mpl.rcParams['lines.{}_pattern'.format(style)])
+        dashes = tuple(mpl.rcParams[f'lines.{style}_pattern'])
     #
     elif isinstance(style, tuple):
         offset, dashes = style
@@ -60,6 +60,18 @@ def _get_dash_pattern(style):
     return offset, dashes
 
 
+def _get_inverse_dash_pattern(offset, dashes):
+    """Return the inverse of the given dash pattern, for filling the gaps."""
+    # Define the inverse pattern by moving the last gap to the start of the
+    # sequence.
+    gaps = dashes[-1:] + dashes[:-1]
+    # Set the offset so that this new first segment is skipped
+    # (see backend_bases.GraphicsContextBase.set_dashes for offset definition).
+    offset_gaps = offset + dashes[-1]
+
+    return offset_gaps, gaps
+
+
 def _scale_dashes(offset, dashes, lw):
     if not mpl.rcParams['lines.scale_dashes']:
         return offset, dashes
@@ -133,7 +145,7 @@ def _mark_every_path(markevery, tpath, affine, ax):
     if isinstance(markevery, tuple):
         if len(markevery) != 2:
             raise ValueError('`markevery` is a tuple but its len is not 2; '
-                             'markevery={}'.format(markevery))
+                             f'markevery={markevery}')
         start, step = markevery
         # if step is an int, old behavior
         if isinstance(step, Integral):
@@ -141,8 +153,8 @@ def _mark_every_path(markevery, tpath, affine, ax):
             if not isinstance(start, Integral):
                 raise ValueError(
                     '`markevery` is a tuple with len 2 and second element is '
-                    'an int, but the first element is not an int; markevery={}'
-                    .format(markevery))
+                    'an int, but the first element is not an int; '
+                    f'markevery={markevery}')
             # just return, we are done here
 
             return Path(verts[slice(start, None, step)],
@@ -153,10 +165,10 @@ def _mark_every_path(markevery, tpath, affine, ax):
                 raise ValueError(
                     '`markevery` is a tuple with len 2 and second element is '
                     'a float, but the first element is not a float or an int; '
-                    'markevery={}'.format(markevery))
+                    f'markevery={markevery}')
             if ax is None:
                 raise ValueError(
-                    "markevery is specified relative to the axes size, but "
+                    "markevery is specified relative to the Axes size, but "
                     "the line does not have a Axes as parent")
 
             # calc cumulative distance along path (in display coords):
@@ -168,7 +180,7 @@ def _mark_every_path(markevery, tpath, affine, ax):
             delta[0, :] = 0
             delta[1:, :] = disp_coords[1:, :] - disp_coords[:-1, :]
             delta = np.hypot(*delta.T).cumsum()
-            # calc distance between markers along path based on the axes
+            # calc distance between markers along path based on the Axes
             # bounding box diagonal being a distance of unity:
             (x0, y0), (x1, y1) = ax.transAxes.transform([[0, 0], [1, 1]])
             scale = np.hypot(x1 - x0, y1 - y0)
@@ -264,15 +276,15 @@ class Line2D(Artist):
         elif self._x is None:
             return "Line2D()"
         elif len(self._x) > 3:
-            return "Line2D((%g,%g),(%g,%g),...,(%g,%g))" % (
-                self._x[0], self._y[0], self._x[0],
-                self._y[0], self._x[-1], self._y[-1])
+            return "Line2D(({:g},{:g}),({:g},{:g}),...,({:g},{:g}))".format(
+                self._x[0], self._y[0],
+                self._x[1], self._y[1],
+                self._x[-1], self._y[-1])
         else:
             return "Line2D(%s)" % ",".join(
                 map("({:g},{:g})".format, self._x, self._y))
 
-    @_api.make_keyword_only("3.6", name="linewidth")
-    def __init__(self, xdata, ydata,
+    def __init__(self, xdata, ydata, *,
                  linewidth=None,  # all Nones default to rc
                  linestyle=None,
                  color=None,
@@ -393,7 +405,7 @@ class Line2D(Artist):
         # update kwargs before updating data to give the caller a
         # chance to init axes (and hence unit support)
         self._internal_update(kwargs)
-        self._pickradius = pickradius
+        self.pickradius = pickradius
         self.ind_offset = 0
         if (isinstance(self._picker, Number) and
                 not isinstance(self._picker, bool)):
@@ -437,9 +449,8 @@ class Line2D(Artist):
 
             TODO: sort returned indices by distance
         """
-        inside, info = self._default_contains(mouseevent)
-        if inside is not None:
-            return inside, info
+        if self._different_canvas(mouseevent):
+            return False, {}
 
         # Make sure we have data to plot
         if self._invalidy or self._invalidx:
@@ -456,11 +467,12 @@ class Line2D(Artist):
         yt = xy[:, 1]
 
         # Convert pick radius from points to pixels
-        if self.figure is None:
+        fig = self.get_figure(root=True)
+        if fig is None:
             _log.warning('no figure set when check if mouse is on line')
             pixels = self._pickradius
         else:
-            pixels = self.figure.dpi / 72. * self._pickradius
+            pixels = fig.dpi / 72. * self._pickradius
 
         # The math involved in checking for containment (here and inside of
         # segment_hits) assumes that it is OK to overflow, so temporarily set
@@ -491,7 +503,6 @@ class Line2D(Artist):
         """
         return self._pickradius
 
-    @_api.rename_parameter("3.6", "d", "pickradius")
     def set_pickradius(self, pickradius):
         """
         Set the pick radius used for containment tests.
@@ -503,7 +514,7 @@ class Line2D(Artist):
         pickradius : float
             Pick radius, in points.
         """
-        if not isinstance(pickradius, Number) or pickradius < 0:
+        if not isinstance(pickradius, Real) or pickradius < 0:
             raise ValueError("pick radius should be a distance")
         self._pickradius = pickradius
 
@@ -565,7 +576,7 @@ class Line2D(Artist):
             - ``every=0.1``, (i.e. a float): markers will be spaced at
               approximately equal visual distances along the line; the distance
               along the line between markers is determined by multiplying the
-              display-coordinate distance of the axes bounding-box diagonal
+              display-coordinate distance of the Axes bounding-box diagonal
               by the value of *every*.
             - ``every=(0.5, 0.1)`` (i.e. a length-2 tuple of float): similar
               to ``every=0.1`` but the first marker will be offset along the
@@ -630,7 +641,7 @@ class Line2D(Artist):
                                  ignore=True)
         # correct for marker size, if any
         if self._marker:
-            ms = (self._markersize / 72.0 * self.figure.dpi) * 0.5
+            ms = (self._markersize / 72.0 * self.get_figure(root=True).dpi) * 0.5
             bbox = bbox.padded(ms)
         return bbox
 
@@ -641,6 +652,11 @@ class Line2D(Artist):
         Parameters
         ----------
         *args : (2, N) array or two 1D arrays
+
+        See Also
+        --------
+        set_xdata
+        set_ydata
         """
         if len(args) == 1:
             (x, y), = args
@@ -778,14 +794,8 @@ class Line2D(Artist):
                     lc_rgba = mcolors.to_rgba(self._gapcolor, self._alpha)
                     gc.set_foreground(lc_rgba, isRGBA=True)
 
-                    # Define the inverse pattern by moving the last gap to the
-                    # start of the sequence.
-                    dashes = self._dash_pattern[1]
-                    gaps = dashes[-1:] + dashes[:-1]
-                    # Set the offset so that this new first segment is skipped
-                    # (see backend_bases.GraphicsContextBase.set_dashes for
-                    # offset definition).
-                    offset_gaps = self._dash_pattern[0] + dashes[-1]
+                    offset_gaps, gaps = _get_inverse_dash_pattern(
+                        *self._dash_pattern)
 
                     gc.set_dashes(offset_gaps, gaps)
                     renderer.draw_path(gc, tpath, affine.frozen())
@@ -1029,9 +1039,7 @@ class Line2D(Artist):
         return self._path
 
     def get_xydata(self):
-        """
-        Return the *xy* data as a Nx2 numpy array.
-        """
+        """Return the *xy* data as a (N, 2) array."""
         if self._invalidy or self._invalidx:
             self.recache()
         return self._xy
@@ -1054,7 +1062,7 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        color : color
+        color : :mpltype:`color`
         """
         mcolors._check_color_like(color=color)
         self._color = color
@@ -1109,7 +1117,7 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        gapcolor : color or None
+        gapcolor : :mpltype:`color` or None
             The color with which to fill the gaps. If None, the gaps are
             unfilled.
         """
@@ -1212,7 +1220,7 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        ec : color
+        ec : :mpltype:`color`
         """
         self._set_markercolor("markeredgecolor", True, ec)
 
@@ -1222,7 +1230,7 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        fc : color
+        fc : :mpltype:`color`
         """
         self._set_markercolor("markerfacecolor", True, fc)
 
@@ -1232,7 +1240,7 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        fc : color
+        fc : :mpltype:`color`
         """
         self._set_markercolor("markerfacecoloralt", False, fc)
 
@@ -1272,16 +1280,14 @@ class Line2D(Artist):
         Parameters
         ----------
         x : 1D array
+
+        See Also
+        --------
+        set_data
+        set_ydata
         """
         if not np.iterable(x):
-            # When deprecation cycle is completed
-            # raise RuntimeError('x must be a sequence')
-            _api.warn_deprecated(
-                since=3.7,
-                message="Setting data with a non sequence type "
-                "is deprecated since %(since)s and will be "
-                "remove %(removal)s")
-            x = [x, ]
+            raise RuntimeError('x must be a sequence')
         self._xorig = copy.copy(x)
         self._invalidx = True
         self.stale = True
@@ -1293,16 +1299,14 @@ class Line2D(Artist):
         Parameters
         ----------
         y : 1D array
+
+        See Also
+        --------
+        set_data
+        set_xdata
         """
         if not np.iterable(y):
-            # When deprecation cycle is completed
-            # raise RuntimeError('y must be a sequence')
-            _api.warn_deprecated(
-                since=3.7,
-                message="Setting data with a non sequence type "
-                "is deprecated since %(since)s and will be "
-                "remove %(removal)s")
-            y = [y, ]
+            raise RuntimeError('y must be a sequence')
         self._yorig = copy.copy(y)
         self._invalidy = True
         self.stale = True
@@ -1462,13 +1466,25 @@ class Line2D(Artist):
         return self._linestyle in ('--', '-.', ':')
 
 
-class _AxLine(Line2D):
+class AxLine(Line2D):
     """
     A helper class that implements `~.Axes.axline`, by recomputing the artist
     transform at draw time.
     """
 
     def __init__(self, xy1, xy2, slope, **kwargs):
+        """
+        Parameters
+        ----------
+        xy1 : (float, float)
+            The first set of (x, y) coordinates for the line to pass through.
+        xy2 : (float, float) or None
+            The second set of (x, y) coordinates for the line to pass through.
+            Both *xy2* and *slope* must be passed, but one of them must be None.
+        slope : float or None
+            The slope of the line. Both *xy2* and *slope* must be passed, but one of
+            them must be None.
+        """
         super().__init__([0, 1], [0, 1], **kwargs)
 
         if (xy2 is None and slope is None or
@@ -1490,8 +1506,8 @@ class _AxLine(Line2D):
                 points_transform.transform([self._xy1, self._xy2])
             dx = x2 - x1
             dy = y2 - y1
-            if np.allclose(x1, x2):
-                if np.allclose(y1, y2):
+            if dx == 0:
+                if dy == 0:
                     raise ValueError(
                         f"Cannot draw a line through two identical points "
                         f"(x={(x1, x2)}, y={(y1, y2)})")
@@ -1505,7 +1521,7 @@ class _AxLine(Line2D):
         (vxlo, vylo), (vxhi, vyhi) = ax.transScale.transform(ax.viewLim)
         # General case: find intersections with view limits in either
         # direction, and draw between the middle two points.
-        if np.isclose(slope, 0):
+        if slope == 0:
             start = vxlo, y1
             stop = vxhi, y1
         elif np.isinf(slope):
@@ -1525,6 +1541,91 @@ class _AxLine(Line2D):
         self._transformed_path = None  # Force regen.
         super().draw(renderer)
 
+    def get_xy1(self):
+        """Return the *xy1* value of the line."""
+        return self._xy1
+
+    def get_xy2(self):
+        """Return the *xy2* value of the line."""
+        return self._xy2
+
+    def get_slope(self):
+        """Return the *slope* value of the line."""
+        return self._slope
+
+    def set_xy1(self, *args, **kwargs):
+        """
+        Set the *xy1* value of the line.
+
+        Parameters
+        ----------
+        xy1 : tuple[float, float]
+            Points for the line to pass through.
+        """
+        params = _api.select_matching_signature([
+            lambda self, x, y: locals(), lambda self, xy1: locals(),
+        ], self, *args, **kwargs)
+        if "x" in params:
+            _api.warn_deprecated("3.10", message=(
+                "Passing x and y separately to AxLine.set_xy1 is deprecated since "
+                "%(since)s; pass them as a single tuple instead."))
+            xy1 = params["x"], params["y"]
+        else:
+            xy1 = params["xy1"]
+        self._xy1 = xy1
+
+    def set_xy2(self, *args, **kwargs):
+        """
+        Set the *xy2* value of the line.
+
+        .. note::
+
+            You can only set *xy2* if the line was created using the *xy2*
+            parameter. If the line was created using *slope*, please use
+            `~.AxLine.set_slope`.
+
+        Parameters
+        ----------
+        xy2 : tuple[float, float]
+            Points for the line to pass through.
+        """
+        if self._slope is None:
+            params = _api.select_matching_signature([
+                lambda self, x, y: locals(), lambda self, xy2: locals(),
+            ], self, *args, **kwargs)
+            if "x" in params:
+                _api.warn_deprecated("3.10", message=(
+                    "Passing x and y separately to AxLine.set_xy2 is deprecated since "
+                    "%(since)s; pass them as a single tuple instead."))
+                xy2 = params["x"], params["y"]
+            else:
+                xy2 = params["xy2"]
+            self._xy2 = xy2
+        else:
+            raise ValueError("Cannot set an 'xy2' value while 'slope' is set;"
+                             " they differ but their functionalities overlap")
+
+    def set_slope(self, slope):
+        """
+        Set the *slope* value of the line.
+
+        .. note::
+
+            You can only set *slope* if the line was created using the *slope*
+            parameter. If the line was created using *xy2*, please use
+            `~.AxLine.set_xy2`.
+
+        Parameters
+        ----------
+        slope : float
+            The slope of the line.
+        """
+        if self._xy2 is None:
+            self._slope = slope
+        else:
+            raise ValueError("Cannot set a 'slope' value while 'xy2' is set;"
+                             " they differ but their functionalities overlap")
+
 
 class VertexSelector:
     """
@@ -1574,7 +1675,7 @@ class VertexSelector:
             'pick_event', self.onpick)
         self.ind = set()
 
-    canvas = property(lambda self: self.axes.figure.canvas)
+    canvas = property(lambda self: self.axes.get_figure(root=True).canvas)
 
     def process_selected(self, ind, xs, ys):
         """
