@@ -1,25 +1,29 @@
 """
 Scales define the distribution of data values on an axis, e.g. a log scaling.
-
-They are attached to an `~.axis.Axis` and hold a `.Transform`, which is
-responsible for the actual data transformation.
+They are defined as subclasses of `ScaleBase`.
 
 See also `.axes.Axes.set_xscale` and the scales examples in the documentation.
+
+See :doc:`/gallery/scales/custom_scale` for a full example of defining a custom
+scale.
+
+Matplotlib also supports non-separable transformations that operate on both
+`~.axis.Axis` at the same time.  They are known as projections, and defined in
+`matplotlib.projections`.
 """
 
 import inspect
 import textwrap
 
 import numpy as np
-from numpy import ma
 
-from matplotlib import cbook, docstring, rcParams
+import matplotlib as mpl
+from matplotlib import _api, _docstring
 from matplotlib.ticker import (
     NullFormatter, ScalarFormatter, LogFormatterSciNotation, LogitFormatter,
     NullLocator, LogLocator, AutoLocator, AutoMinorLocator,
-    SymmetricalLogLocator, LogitLocator)
+    SymmetricalLogLocator, AsinhLocator, LogitLocator)
 from matplotlib.transforms import Transform, IdentityTransform
-from matplotlib.cbook import warn_deprecated
 
 
 class ScaleBase:
@@ -28,44 +32,39 @@ class ScaleBase:
 
     Scales are separable transformations, working on a single dimension.
 
-    Any subclasses will want to override:
-
-    - :attr:`name`
-    - :meth:`get_transform`
-    - :meth:`set_default_locators_and_formatters`
-
-    And optionally:
-
-    - :meth:`limit_range_for_scale`
+    Subclasses should override
 
+    :attr:`name`
+        The scale's name.
+    :meth:`get_transform`
+        A method returning a `.Transform`, which converts data coordinates to
+        scaled coordinates.  This transform should be invertible, so that e.g.
+        mouse positions can be converted back to data coordinates.
+    :meth:`set_default_locators_and_formatters`
+        A method that sets default locators and formatters for an `~.axis.Axis`
+        that uses this scale.
+    :meth:`limit_range_for_scale`
+        An optional method that "fixes" the axis range to acceptable values,
+        e.g. restricting log-scaled axes to positive values.
     """
 
-    def __init__(self, axis, **kwargs):
+    def __init__(self, axis):
         r"""
         Construct a new scale.
 
         Notes
         -----
-        The following note is for scale implementors.
+        The following note is for scale implementers.
 
         For back-compatibility reasons, scales take an `~matplotlib.axis.Axis`
         object as first argument.  However, this argument should not
         be used: a single scale object should be usable by multiple
         `~matplotlib.axis.Axis`\es at the same time.
         """
-        if kwargs:
-            warn_deprecated(
-                '3.2.0',
-                message=(
-                    f"ScaleBase got an unexpected keyword "
-                    f"argument {next(iter(kwargs))!r}. "
-                    'In the future this will raise TypeError')
-            )
 
     def get_transform(self):
         """
-        Return the :class:`~matplotlib.transforms.Transform` object
-        associated with this scale.
+        Return the `.Transform` object associated with this scale.
         """
         raise NotImplementedError()
 
@@ -78,8 +77,8 @@ class ScaleBase:
 
     def limit_range_for_scale(self, vmin, vmax, minpos):
         """
-        Returns the range *vmin*, *vmax*, possibly limited to the
-        domain supported by this scale.
+        Return the range *vmin*, *vmax*, restricted to the
+        domain supported by this scale (if any).
 
         *minpos* should be the minimum positive value in the data.
         This is used by log scales to determine a minimum value.
@@ -94,13 +93,12 @@ class LinearScale(ScaleBase):
 
     name = 'linear'
 
-    def __init__(self, axis, **kwargs):
+    def __init__(self, axis):
         # This method is present only to prevent inheritance of the base class'
         # constructor docstring, which would otherwise end up interpolated into
         # the docstring of Axis.set_scale.
         """
-        """
-        super().__init__(axis, **kwargs)
+        """  # noqa: D419
 
     def set_default_locators_and_formatters(self, axis):
         # docstring inherited
@@ -108,8 +106,8 @@ class LinearScale(ScaleBase):
         axis.set_major_formatter(ScalarFormatter())
         axis.set_minor_formatter(NullFormatter())
         # update the minor locator for x and y axis based on rcParams
-        if (axis.axis_name == 'x' and rcParams['xtick.minor.visible']
-            or axis.axis_name == 'y' and rcParams['ytick.minor.visible']):
+        if (axis.axis_name == 'x' and mpl.rcParams['xtick.minor.visible'] or
+                axis.axis_name == 'y' and mpl.rcParams['ytick.minor.visible']):
             axis.set_minor_locator(AutoMinorLocator())
         else:
             axis.set_minor_locator(NullLocator())
@@ -193,108 +191,36 @@ class FuncScale(ScaleBase):
         axis.set_major_formatter(ScalarFormatter())
         axis.set_minor_formatter(NullFormatter())
         # update the minor locator for x and y axis based on rcParams
-        if (axis.axis_name == 'x' and rcParams['xtick.minor.visible']
-            or axis.axis_name == 'y' and rcParams['ytick.minor.visible']):
+        if (axis.axis_name == 'x' and mpl.rcParams['xtick.minor.visible'] or
+                axis.axis_name == 'y' and mpl.rcParams['ytick.minor.visible']):
             axis.set_minor_locator(AutoMinorLocator())
         else:
             axis.set_minor_locator(NullLocator())
 
 
-@cbook.deprecated("3.1", alternative="LogTransform")
-class LogTransformBase(Transform):
-    input_dims = output_dims = 1
-
-    def __init__(self, nonpos='clip'):
-        Transform.__init__(self)
-        self._clip = {"clip": True, "mask": False}[nonpos]
-
-    def transform_non_affine(self, a):
-        return LogTransform.transform_non_affine(self, a)
-
-    def __str__(self):
-        return "{}({!r})".format(
-            type(self).__name__, "clip" if self._clip else "mask")
-
-
-@cbook.deprecated("3.1", alternative="InvertedLogTransform")
-class InvertedLogTransformBase(Transform):
-    input_dims = output_dims = 1
-
-    def transform_non_affine(self, a):
-        return ma.power(self.base, a)
-
-    def __str__(self):
-        return "{}()".format(type(self).__name__)
-
-
-@cbook.deprecated("3.1", alternative="LogTransform")
-class Log10Transform(LogTransformBase):
-    base = 10.0
-
-    def inverted(self):
-        return InvertedLog10Transform()
-
-
-@cbook.deprecated("3.1", alternative="InvertedLogTransform")
-class InvertedLog10Transform(InvertedLogTransformBase):
-    base = 10.0
-
-    def inverted(self):
-        return Log10Transform()
-
-
-@cbook.deprecated("3.1", alternative="LogTransform")
-class Log2Transform(LogTransformBase):
-    base = 2.0
-
-    def inverted(self):
-        return InvertedLog2Transform()
-
-
-@cbook.deprecated("3.1", alternative="InvertedLogTransform")
-class InvertedLog2Transform(InvertedLogTransformBase):
-    base = 2.0
-
-    def inverted(self):
-        return Log2Transform()
-
-
-@cbook.deprecated("3.1", alternative="LogTransform")
-class NaturalLogTransform(LogTransformBase):
-    base = np.e
-
-    def inverted(self):
-        return InvertedNaturalLogTransform()
-
-
-@cbook.deprecated("3.1", alternative="InvertedLogTransform")
-class InvertedNaturalLogTransform(InvertedLogTransformBase):
-    base = np.e
-
-    def inverted(self):
-        return NaturalLogTransform()
-
-
 class LogTransform(Transform):
     input_dims = output_dims = 1
 
-    def __init__(self, base, nonpos='clip'):
-        Transform.__init__(self)
+    def __init__(self, base, nonpositive='clip'):
+        super().__init__()
+        if base <= 0 or base == 1:
+            raise ValueError('The log base cannot be <= 0 or == 1')
         self.base = base
-        self._clip = {"clip": True, "mask": False}[nonpos]
+        self._clip = _api.check_getitem(
+            {"clip": True, "mask": False}, nonpositive=nonpositive)
 
     def __str__(self):
-        return "{}(base={}, nonpos={!r})".format(
+        return "{}(base={}, nonpositive={!r})".format(
             type(self).__name__, self.base, "clip" if self._clip else "mask")
 
-    def transform_non_affine(self, a):
+    def transform_non_affine(self, values):
         # Ignore invalid values due to nans being passed to the transform.
         with np.errstate(divide="ignore", invalid="ignore"):
             log = {np.e: np.log, 2: np.log2, 10: np.log10}.get(self.base)
             if log:  # If possible, do everything in a single call to NumPy.
-                out = log(a)
+                out = log(values)
             else:
-                out = np.log(a)
+                out = np.log(values)
                 out /= np.log(self.base)
             if self._clip:
                 # SVG spec says that conforming viewers must support values up
@@ -306,7 +232,7 @@ class LogTransform(Transform):
                 # pass. On the other hand, in practice, we want to clip beyond
                 #     np.log10(np.nextafter(0, 1)) ~ -323
                 # so 1000 seems safe.
-                out[a <= 0] = -1000
+                out[values <= 0] = -1000
         return out
 
     def inverted(self):
@@ -317,14 +243,14 @@ class InvertedLogTransform(Transform):
     input_dims = output_dims = 1
 
     def __init__(self, base):
-        Transform.__init__(self)
+        super().__init__()
         self.base = base
 
     def __str__(self):
-        return "{}(base={})".format(type(self).__name__, self.base)
+        return f"{type(self).__name__}(base={self.base})"
 
-    def transform_non_affine(self, a):
-        return ma.power(self.base, a)
+    def transform_non_affine(self, values):
+        return np.power(self.base, values)
 
     def inverted(self):
         return LogTransform(self.base)
@@ -336,58 +262,26 @@ class LogScale(ScaleBase):
     """
     name = 'log'
 
-    # compatibility shim
-    LogTransformBase = LogTransformBase
-    Log10Transform = Log10Transform
-    InvertedLog10Transform = InvertedLog10Transform
-    Log2Transform = Log2Transform
-    InvertedLog2Transform = InvertedLog2Transform
-    NaturalLogTransform = NaturalLogTransform
-    InvertedNaturalLogTransform = InvertedNaturalLogTransform
-    LogTransform = LogTransform
-    InvertedLogTransform = InvertedLogTransform
-
-    def __init__(self, axis, **kwargs):
+    def __init__(self, axis, *, base=10, subs=None, nonpositive="clip"):
         """
         Parameters
         ----------
         axis : `~matplotlib.axis.Axis`
             The axis for the scale.
-        basex, basey : float, default: 10
+        base : float, default: 10
             The base of the logarithm.
-        nonposx, nonposy : {'clip', 'mask'}, default: 'clip'
+        nonpositive : {'clip', 'mask'}, default: 'clip'
             Determines the behavior for non-positive values. They can either
             be masked as invalid, or clipped to a very small positive number.
-        subsx, subsy : sequence of int, default: None
-            Where to place the subticks between each major tick.
-            For example, in a log10 scale: ``[2, 3, 4, 5, 6, 7, 8, 9]``
-            will place 8 logarithmically spaced minor ticks between
-            each major tick.
+        subs : sequence of int, default: None
+            Where to place the subticks between each major tick.  For example,
+            in a log10 scale, ``[2, 3, 4, 5, 6, 7, 8, 9]`` will place 8
+            logarithmically spaced minor ticks between each major tick.
         """
-        if axis.axis_name == 'x':
-            base = kwargs.pop('basex', 10.0)
-            subs = kwargs.pop('subsx', None)
-            nonpos = kwargs.pop('nonposx', 'clip')
-            cbook._check_in_list(['mask', 'clip'], nonposx=nonpos)
-        else:
-            base = kwargs.pop('basey', 10.0)
-            subs = kwargs.pop('subsy', None)
-            nonpos = kwargs.pop('nonposy', 'clip')
-            cbook._check_in_list(['mask', 'clip'], nonposy=nonpos)
-
-        if kwargs:
-            raise TypeError(f"LogScale got an unexpected keyword "
-                            f"argument {next(iter(kwargs))!r}")
-
-        if base <= 0 or base == 1:
-            raise ValueError('The log base cannot be <= 0 or == 1')
-
-        self._transform = LogTransform(base, nonpos)
+        self._transform = LogTransform(base, nonpositive)
         self.subs = subs
 
-    @property
-    def base(self):
-        return self._transform.base
+    base = property(lambda self: self._transform.base)
 
     def set_default_locators_and_formatters(self, axis):
         # docstring inherited
@@ -423,7 +317,7 @@ class FuncScaleLog(LogScale):
         """
         Parameters
         ----------
-        axis : `matplotlib.axis.Axis`
+        axis : `~matplotlib.axis.Axis`
             The axis for the scale.
         functions : (callable, callable)
             two-tuple of the forward and inverse functions for the scale.
@@ -433,9 +327,8 @@ class FuncScaleLog(LogScale):
 
                 def forward(values: array-like) -> array-like
 
-        base : float
-            logarithmic base of the scale (default = 10)
-
+        base : float, default: 10
+            Logarithmic base of the scale.
         """
         forward, inverse = functions
         self.subs = None
@@ -454,21 +347,27 @@ class SymmetricalLogTransform(Transform):
     input_dims = output_dims = 1
 
     def __init__(self, base, linthresh, linscale):
-        Transform.__init__(self)
+        super().__init__()
+        if base <= 1.0:
+            raise ValueError("'base' must be larger than 1")
+        if linthresh <= 0.0:
+            raise ValueError("'linthresh' must be positive")
+        if linscale <= 0.0:
+            raise ValueError("'linscale' must be positive")
         self.base = base
         self.linthresh = linthresh
         self.linscale = linscale
         self._linscale_adj = (linscale / (1.0 - self.base ** -1))
         self._log_base = np.log(base)
 
-    def transform_non_affine(self, a):
-        abs_a = np.abs(a)
+    def transform_non_affine(self, values):
+        abs_a = np.abs(values)
         with np.errstate(divide="ignore", invalid="ignore"):
-            out = np.sign(a) * self.linthresh * (
+            out = np.sign(values) * self.linthresh * (
                 self._linscale_adj +
                 np.log(abs_a / self.linthresh) / self._log_base)
             inside = abs_a <= self.linthresh
-        out[inside] = a[inside] * self._linscale_adj
+        out[inside] = values[inside] * self._linscale_adj
         return out
 
     def inverted(self):
@@ -480,7 +379,7 @@ class InvertedSymmetricalLogTransform(Transform):
     input_dims = output_dims = 1
 
     def __init__(self, base, linthresh, linscale):
-        Transform.__init__(self)
+        super().__init__()
         symlog = SymmetricalLogTransform(base, linthresh, linscale)
         self.base = base
         self.linthresh = linthresh
@@ -488,14 +387,14 @@ class InvertedSymmetricalLogTransform(Transform):
         self.linscale = linscale
         self._linscale_adj = (linscale / (1.0 - self.base ** -1))
 
-    def transform_non_affine(self, a):
-        abs_a = np.abs(a)
+    def transform_non_affine(self, values):
+        abs_a = np.abs(values)
         with np.errstate(divide="ignore", invalid="ignore"):
-            out = np.sign(a) * self.linthresh * (
+            out = np.sign(values) * self.linthresh * (
                 np.power(self.base,
                          abs_a / self.linthresh - self._linscale_adj))
             inside = abs_a <= self.invlinthresh
-        out[inside] = a[inside] / self._linscale_adj
+        out[inside] = values[inside] / self._linscale_adj
         return out
 
     def inverted(self):
@@ -515,19 +414,19 @@ class SymmetricalLogScale(ScaleBase):
 
     Parameters
     ----------
-    basex, basey : float
-        The base of the logarithm. Defaults to 10.
+    base : float, default: 10
+        The base of the logarithm.
 
-    linthreshx, linthreshy : float
+    linthresh : float, default: 2
         Defines the range ``(-x, x)``, within which the plot is linear.
-        This avoids having the plot go to infinity around zero. Defaults to 2.
+        This avoids having the plot go to infinity around zero.
 
-    subsx, subsy : sequence of int
+    subs : sequence of int
         Where to place the subticks between each major tick.
         For example, in a log10 scale: ``[2, 3, 4, 5, 6, 7, 8, 9]`` will place
         8 logarithmically spaced minor ticks between each major tick.
 
-    linscalex, linscaley : float, optional
+    linscale : float, optional
         This allows the linear range ``(-linthresh, linthresh)`` to be
         stretched relative to the logarithmic range. Its value is the number of
         decades to use for each half of the linear range. For example, when
@@ -536,45 +435,15 @@ class SymmetricalLogScale(ScaleBase):
         the logarithmic range.
     """
     name = 'symlog'
-    # compatibility shim
-    SymmetricalLogTransform = SymmetricalLogTransform
-    InvertedSymmetricalLogTransform = InvertedSymmetricalLogTransform
-
-    def __init__(self, axis, **kwargs):
-        if axis.axis_name == 'x':
-            base = kwargs.pop('basex', 10.0)
-            linthresh = kwargs.pop('linthreshx', 2.0)
-            subs = kwargs.pop('subsx', None)
-            linscale = kwargs.pop('linscalex', 1.0)
-        else:
-            base = kwargs.pop('basey', 10.0)
-            linthresh = kwargs.pop('linthreshy', 2.0)
-            subs = kwargs.pop('subsy', None)
-            linscale = kwargs.pop('linscaley', 1.0)
-        if kwargs:
-            warn_deprecated(
-                '3.2.0',
-                message=(
-                    f"SymmetricalLogScale got an unexpected keyword "
-                    f"argument {next(iter(kwargs))!r}. "
-                    'In the future this will raise TypeError')
-            )
-            # raise TypeError(f"SymmetricalLogScale got an unexpected keyword "
-            #                 f"argument {next(iter(kwargs))!r}")
-
-        if base <= 1.0:
-            raise ValueError("'basex/basey' must be larger than 1")
-        if linthresh <= 0.0:
-            raise ValueError("'linthreshx/linthreshy' must be positive")
-        if linscale <= 0.0:
-            raise ValueError("'linscalex/linthreshy' must be positive")
 
+    def __init__(self, axis, *, base=10, linthresh=2, subs=None, linscale=1):
         self._transform = SymmetricalLogTransform(base, linthresh, linscale)
-        self.base = base
-        self.linthresh = linthresh
-        self.linscale = linscale
         self.subs = subs
 
+    base = property(lambda self: self._transform.base)
+    linthresh = property(lambda self: self._transform.linthresh)
+    linscale = property(lambda self: self._transform.linscale)
+
     def set_default_locators_and_formatters(self, axis):
         # docstring inherited
         axis.set_major_locator(SymmetricalLogLocator(self.get_transform()))
@@ -588,47 +457,164 @@ class SymmetricalLogScale(ScaleBase):
         return self._transform
 
 
+class AsinhTransform(Transform):
+    """Inverse hyperbolic-sine transformation used by `.AsinhScale`"""
+    input_dims = output_dims = 1
+
+    def __init__(self, linear_width):
+        super().__init__()
+        if linear_width <= 0.0:
+            raise ValueError("Scale parameter 'linear_width' " +
+                             "must be strictly positive")
+        self.linear_width = linear_width
+
+    def transform_non_affine(self, values):
+        return self.linear_width * np.arcsinh(values / self.linear_width)
+
+    def inverted(self):
+        return InvertedAsinhTransform(self.linear_width)
+
+
+class InvertedAsinhTransform(Transform):
+    """Hyperbolic sine transformation used by `.AsinhScale`"""
+    input_dims = output_dims = 1
+
+    def __init__(self, linear_width):
+        super().__init__()
+        self.linear_width = linear_width
+
+    def transform_non_affine(self, values):
+        return self.linear_width * np.sinh(values / self.linear_width)
+
+    def inverted(self):
+        return AsinhTransform(self.linear_width)
+
+
+class AsinhScale(ScaleBase):
+    """
+    A quasi-logarithmic scale based on the inverse hyperbolic sine (asinh)
+
+    For values close to zero, this is essentially a linear scale,
+    but for large magnitude values (either positive or negative)
+    it is asymptotically logarithmic. The transition between these
+    linear and logarithmic regimes is smooth, and has no discontinuities
+    in the function gradient in contrast to
+    the `.SymmetricalLogScale` ("symlog") scale.
+
+    Specifically, the transformation of an axis coordinate :math:`a` is
+    :math:`a \\rightarrow a_0 \\sinh^{-1} (a / a_0)` where :math:`a_0`
+    is the effective width of the linear region of the transformation.
+    In that region, the transformation is
+    :math:`a \\rightarrow a + \\mathcal{O}(a^3)`.
+    For large values of :math:`a` the transformation behaves as
+    :math:`a \\rightarrow a_0 \\, \\mathrm{sgn}(a) \\ln |a| + \\mathcal{O}(1)`.
+
+    .. note::
+
+       This API is provisional and may be revised in the future
+       based on early user feedback.
+    """
+
+    name = 'asinh'
+
+    auto_tick_multipliers = {
+        3: (2, ),
+        4: (2, ),
+        5: (2, ),
+        8: (2, 4),
+        10: (2, 5),
+        16: (2, 4, 8),
+        64: (4, 16),
+        1024: (256, 512)
+    }
+
+    def __init__(self, axis, *, linear_width=1.0,
+                 base=10, subs='auto', **kwargs):
+        """
+        Parameters
+        ----------
+        linear_width : float, default: 1
+            The scale parameter (elsewhere referred to as :math:`a_0`)
+            defining the extent of the quasi-linear region,
+            and the coordinate values beyond which the transformation
+            becomes asymptotically logarithmic.
+        base : int, default: 10
+            The number base used for rounding tick locations
+            on a logarithmic scale. If this is less than one,
+            then rounding is to the nearest integer multiple
+            of powers of ten.
+        subs : sequence of int
+            Multiples of the number base used for minor ticks.
+            If set to 'auto', this will use built-in defaults,
+            e.g. (2, 5) for base=10.
+        """
+        super().__init__(axis)
+        self._transform = AsinhTransform(linear_width)
+        self._base = int(base)
+        if subs == 'auto':
+            self._subs = self.auto_tick_multipliers.get(self._base)
+        else:
+            self._subs = subs
+
+    linear_width = property(lambda self: self._transform.linear_width)
+
+    def get_transform(self):
+        return self._transform
+
+    def set_default_locators_and_formatters(self, axis):
+        axis.set(major_locator=AsinhLocator(self.linear_width,
+                                            base=self._base),
+                 minor_locator=AsinhLocator(self.linear_width,
+                                            base=self._base,
+                                            subs=self._subs),
+                 minor_formatter=NullFormatter())
+        if self._base > 1:
+            axis.set_major_formatter(LogFormatterSciNotation(self._base))
+        else:
+            axis.set_major_formatter('{x:.3g}')
+
+
 class LogitTransform(Transform):
     input_dims = output_dims = 1
 
-    def __init__(self, nonpos='mask'):
-        Transform.__init__(self)
-        cbook._check_in_list(['mask', 'clip'], nonpos=nonpos)
-        self._nonpos = nonpos
-        self._clip = {"clip": True, "mask": False}[nonpos]
+    def __init__(self, nonpositive='mask'):
+        super().__init__()
+        _api.check_in_list(['mask', 'clip'], nonpositive=nonpositive)
+        self._nonpositive = nonpositive
+        self._clip = {"clip": True, "mask": False}[nonpositive]
 
-    def transform_non_affine(self, a):
+    def transform_non_affine(self, values):
         """logit transform (base 10), masked or clipped"""
         with np.errstate(divide="ignore", invalid="ignore"):
-            out = np.log10(a / (1 - a))
+            out = np.log10(values / (1 - values))
         if self._clip:  # See LogTransform for choice of clip value.
-            out[a <= 0] = -1000
-            out[1 <= a] = 1000
+            out[values <= 0] = -1000
+            out[1 <= values] = 1000
         return out
 
     def inverted(self):
-        return LogisticTransform(self._nonpos)
+        return LogisticTransform(self._nonpositive)
 
     def __str__(self):
-        return "{}({!r})".format(type(self).__name__, self._nonpos)
+        return f"{type(self).__name__}({self._nonpositive!r})"
 
 
 class LogisticTransform(Transform):
     input_dims = output_dims = 1
 
-    def __init__(self, nonpos='mask'):
-        Transform.__init__(self)
-        self._nonpos = nonpos
+    def __init__(self, nonpositive='mask'):
+        super().__init__()
+        self._nonpositive = nonpositive
 
-    def transform_non_affine(self, a):
+    def transform_non_affine(self, values):
         """logistic transform (base 10)"""
-        return 1.0 / (1 + 10**(-a))
+        return 1.0 / (1 + 10**(-values))
 
     def inverted(self):
-        return LogitTransform(self._nonpos)
+        return LogitTransform(self._nonpositive)
 
     def __str__(self):
-        return "{}({!r})".format(type(self).__name__, self._nonpos)
+        return f"{type(self).__name__}({self._nonpositive!r})"
 
 
 class LogitScale(ScaleBase):
@@ -640,20 +626,14 @@ class LogitScale(ScaleBase):
     """
     name = 'logit'
 
-    def __init__(
-        self,
-        axis,
-        nonpos='mask',
-        *,
-        one_half=r"\frac{1}{2}",
-        use_overline=False,
-    ):
+    def __init__(self, axis, nonpositive='mask', *,
+                 one_half=r"\frac{1}{2}", use_overline=False):
         r"""
         Parameters
         ----------
-        axis : `matplotlib.axis.Axis`
+        axis : `~matplotlib.axis.Axis`
             Currently unused.
-        nonpos : {'mask', 'clip'}
+        nonpositive : {'mask', 'clip'}
             Determines the behavior for values beyond the open interval ]0, 1[.
             They can either be masked as invalid, or clipped to a number very
             close to 0 or 1.
@@ -663,7 +643,7 @@ class LogitScale(ScaleBase):
         one_half : str, default: r"\frac{1}{2}"
             The string used for ticks formatter to represent 1/2.
         """
-        self._transform = LogitTransform(nonpos)
+        self._transform = LogitTransform(nonpositive)
         self._use_overline = use_overline
         self._one_half = one_half
 
@@ -704,6 +684,7 @@ _scale_mapping = {
     'linear': LinearScale,
     'log':    LogScale,
     'symlog': SymmetricalLogScale,
+    'asinh':  AsinhScale,
     'logit':  LogitScale,
     'function': FuncScale,
     'functionlog': FuncScaleLog,
@@ -722,11 +703,10 @@ def scale_factory(scale, axis, **kwargs):
     Parameters
     ----------
     scale : {%(names)s}
-    axis : `matplotlib.axis.Axis`
+    axis : `~matplotlib.axis.Axis`
     """
-    scale = scale.lower()
-    cbook._check_in_list(_scale_mapping, scale=scale)
-    return _scale_mapping[scale](axis, **kwargs)
+    scale_cls = _api.check_getitem(_scale_mapping, scale=scale)
+    return scale_cls(axis, **kwargs)
 
 
 if scale_factory.__doc__:
@@ -746,32 +726,23 @@ def register_scale(scale_class):
     _scale_mapping[scale_class.name] = scale_class
 
 
-@cbook.deprecated(
-    '3.1', message='get_scale_docs() is considered private API since '
-                   '3.1 and will be removed from the public API in 3.3.')
-def get_scale_docs():
-    """
-    Helper function for generating docstrings related to scales.
-    """
-    return _get_scale_docs()
-
-
 def _get_scale_docs():
     """
     Helper function for generating docstrings related to scales.
     """
     docs = []
     for name, scale_class in _scale_mapping.items():
+        docstring = inspect.getdoc(scale_class.__init__) or ""
         docs.extend([
             f"    {name!r}",
             "",
-            textwrap.indent(inspect.getdoc(scale_class.__init__), " " * 8),
+            textwrap.indent(docstring, " " * 8),
             ""
         ])
     return "\n".join(docs)
 
 
-docstring.interpd.update(
+_docstring.interpd.register(
     scale_type='{%s}' % ', '.join([repr(x) for x in get_scale_names()]),
     scale_docs=_get_scale_docs().rstrip(),
     )
